import numpy as np
from mpmath import mp

def Cij_0(eigvec, Np):
	Cij=eigvec[:,0:Np]*eigvec[:,0:Np].T
	return(Cij)

def FreeFermions(subsystem,C_t):
	C=mp.matrix([[C_t[x,y] for x in subsystem] for y in subsystem])
	C_eigval=mp.eigh(C, eigvals_only=True)
	EH_eigval=mp.matrix([mp.log(mp.fdiv(mp.fsub(mp.mpf(1.0),x),x)) for x in C_eigval])
	S=mp.re(mp.fsum([mp.log(mp.mpf(1.0)+mp.exp(-x))+mp.fdiv(x,mp.exp(x)+mp.mpf(1.0)) for x in EH_eigval]))
	return(S)
	
ClosedLoop = False

t_vec= np.arange(0.0, 40, .1)#vector: times


v1=1.5
v2=0.1

L_vec=[16, 32, 64, 128]

for L in L_vec: 
	Np = int(L / 2)
	l=[int(L/4), int(L/2), int(3*L/4), int(L)]
	A = list(range(l[0]))
	B = list(range(l[0],l[1]))
	D = list(range(l[1],l[2]))
	C = list(range(l[2],l[3]))
	

	outputfile="./Sq_quench_L="+str(L)+"_v1="+mp.nstr(v1,2)+"_v2="+mp.nstr(v2,2)+".dat"#file in which I will write the results
	
	D_alternative=A+B+C
	if len(D)>len(D_alternative):
		D=D_alternative

	f1=open(outputfile,"w")
		
	mp.dps=L*3 #sets the digits of the decimal numbers
	
	END = L - 1

	if ClosedLoop == True:
		END = L

	#buildHamiltonian
	H1=mp.matrix(L)
	H2=mp.matrix(L)
	#the following cicle takes care of the staggered hopping
	for i in range(END):
		j = (i + 1) % L
		if i % 2 == 0:
			H1[i, j] = -v1
			H1[j, i] = H1[i, j]
			H2[i, j] = -v2
			H2[j, i] = H2[i, j]
		elif j % 2 == 0:
			H1[i, j] = -1
			H1[j, i] = H1[i, j]
			H2[i, j] = -1
			H2[j, i] = H2[i, j]
						

	#find its eigenvalues and vectors
	eigval1, eigvec1=mp.eigh(H1)
	eigval1, eigvec1=mp.eig_sort(eigval1,eigvec1)
	eigval2, eigvec2=mp.eigh(H2)
	eigval2, eigvec2=mp.eig_sort(eigval2,eigvec2)
		
	#From now on I will use the Free Fermions technique to calculate the entanglement entropies of the subsistems in units of log2
	Cij_in=Cij_0(eigvec1,Np)
		
	for t in t_vec:
		print(mp.nstr(v1,2)+"------>")
		print("Currently working on...")
		print("L="+mp.nstr(L,1)+" at time "+ mp.nstr(t,2))
		
		iteration=0 #this counts the iteration on the 
		
		phases=mp.diag([mp.exp(mp.j*x*t) for x in eigval2])
		psi=eigvec2*phases
		#print(eigvec2*eigvec2.T)
		
		M=eigvec2.conjugate()*(psi).T
		#print(M)
		C_t=M*Cij_in*(M.H)
			
		SB=FreeFermions(B,C_t)/mp.log(mp.mpf(2.0))
			
		SAB=FreeFermions(A+B,C_t)/mp.log(mp.mpf(2.0))
			
		SBC=FreeFermions(B+C,C_t)/mp.log(mp.mpf(2.0))
						
		SABC=FreeFermions(D,C_t)/mp.log(mp.mpf(2.0)) 
			
			
			
		#In the end I calculate Sqtopo ad proposed by Wen
		Sq=(SAB+SBC-SB-SABC)
		
		print(mp.nstr(Sq,2))

		f1.write ("%.5f,%.40f,%.40f,%.40f,%.40f,%.40f\n" %(t,SAB,SBC,SB,SABC, Sq))#print to file

	f1.close()

